@@ -183,8 +183,7 @@ librdf_new_hash_datum(librdf_world *world, void *data, size_t size)
   if((datum = world->hash_datums_list)) {
     world->hash_datums_list = datum->next;
   } else {
-    datum = (librdf_hash_datum*)LIBRDF_CALLOC(librdf_hash_datum, 1,
-                                              sizeof(librdf_hash_datum));
+    datum = LIBRDF_CALLOC(librdf_hash_datum*, 1, sizeof(*datum));
     if(datum)
       datum->world = world;
   }
@@ -212,11 +211,14 @@ librdf_new_hash_datum(librdf_world *world, void *data, size_t size)
 void
 librdf_free_hash_datum(librdf_hash_datum *datum) 
 {
+  if(!datum)
+    return;
+  
   if(datum->data) {
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
     LIBRDF_DEBUG2("Freeing datum data, %p\n", datum);
 #endif
-    LIBRDF_FREE(cstring, datum->data);
+    LIBRDF_FREE(char*, datum->data);
     datum->data = NULL;
   }
 
@@ -265,12 +267,11 @@ librdf_hash_register_factory(librdf_world *world, const char *name,
     }
   }
 
-  hash=(librdf_hash_factory*)LIBRDF_CALLOC(librdf_hash_factory, 1,
-                                           sizeof(librdf_hash_factory));
+  hash = LIBRDF_CALLOC(librdf_hash_factory*, 1, sizeof(*hash));
   if(!hash)
     goto oom;
   
-  hash->name=(char*)LIBRDF_MALLOC(cstring, strlen(name)+1);
+  hash->name = LIBRDF_MALLOC(char*, strlen(name) + 1);
   if(!hash->name)
     goto oom_tidy;
   strcpy(hash->name, name);
@@ -377,12 +378,11 @@ librdf_new_hash_from_factory(librdf_world *world,
 
   librdf_world_open(world);
 
-  h=(librdf_hash*)LIBRDF_CALLOC(librdf_hash, sizeof(librdf_hash), 1);
+  h = LIBRDF_CALLOC(librdf_hash*, 1, sizeof(*h));
   if(!h)
     return NULL;
   
-  h->context=(char*)LIBRDF_CALLOC(librdf_hash_context, 1,
-                                  factory->context_length);
+  h->context = LIBRDF_CALLOC(void*, 1, factory->context_length);
   if(!h->context) {
     librdf_free_hash(h);
     return NULL;
@@ -479,15 +479,14 @@ librdf_new_hash_from_hash(librdf_hash* old_hash)
 {
   librdf_hash* hash;
   
-  hash=(librdf_hash*)LIBRDF_CALLOC(librdf_hash, sizeof(librdf_hash), 1);
+  hash = LIBRDF_CALLOC(librdf_hash*, 1, sizeof(*hash));
   if(!hash)
     return NULL;
 
   hash->world=old_hash->world;
   hash->factory=old_hash->factory;
 
-  hash->context=(char*)LIBRDF_CALLOC(librdf_hash_context, 1,
-                                     hash->factory->context_length);
+  hash->context = LIBRDF_CALLOC(void*, 1, hash->factory->context_length);
   if(!hash->context) {
     librdf_free_hash(hash);
     return NULL;
@@ -504,7 +503,7 @@ librdf_new_hash_from_hash(librdf_hash* old_hash)
   if(hash->factory->clone(hash, hash->context, hash->identifier,
                           old_hash->context)) {
     if(hash->identifier)
-      LIBRDF_FREE(cstring, hash->identifier);
+      LIBRDF_FREE(char*, hash->identifier);
     librdf_free_hash(hash);
     return NULL;
   }
@@ -522,6 +521,9 @@ librdf_new_hash_from_hash(librdf_hash* old_hash)
 void
 librdf_free_hash(librdf_hash* hash) 
 {
+  if(!hash)
+    return;
+  
   if(hash->context) {
     if(hash->is_open)
       librdf_hash_close(hash);
@@ -558,7 +560,7 @@ librdf_hash_open(librdf_hash* hash, const char *identifier,
   int status;
 
   if(identifier) {
-    hash->identifier=(char*)LIBRDF_MALLOC(cstring, strlen(identifier)+1);
+    hash->identifier = LIBRDF_MALLOC(char*, strlen(identifier) + 1);
     if(!hash->identifier)
       return 1;
     strcpy(hash->identifier, identifier);
@@ -585,7 +587,7 @@ librdf_hash_close(librdf_hash* hash)
 {
   hash->is_open=0;
   if(hash->identifier) {
-    LIBRDF_FREE(cstring,hash->identifier);
+    LIBRDF_FREE(char*, hash->identifier);
     hash->identifier=NULL;
   }
   return hash->factory->close(hash->context);
@@ -633,7 +635,7 @@ librdf_hash_get(librdf_hash* hash, const char *key)
 
   if(hd_value) {
     if(hd_value->data) {
-      value=(char*)LIBRDF_MALLOC(cstring, hd_value->size+1);
+      value = LIBRDF_MALLOC(char*, hd_value->size + 1);
       if(value) {
         /* Copy into new null terminated string for userland */
         memcpy(value, hd_value->data, hd_value->size);
@@ -684,7 +686,7 @@ librdf_hash_get_one(librdf_hash* hash, librdf_hash_datum *key)
   status=librdf_hash_cursor_get_next(cursor, key, value);
   if(!status) {
     /* value->data will point to SHARED area, so copy it */
-    new_value=(char*)LIBRDF_MALLOC(cstring, value->size);
+    new_value = LIBRDF_MALLOC(char*, value->size);
     if(new_value) {
       memcpy(new_value, value->data, value->size);
       value->data=new_value;
@@ -741,7 +743,8 @@ librdf_hash_get_all(librdf_hash* hash,
   int status;
   librdf_iterator* iterator;
   
-  context=(librdf_hash_get_all_iterator_context*)LIBRDF_CALLOC(librdf_hash_get_all_iterator_context, 1, sizeof(librdf_hash_get_all_iterator_context));
+  context = LIBRDF_CALLOC(librdf_hash_get_all_iterator_context*, 1,
+                          sizeof(*context));
   if(!context)
     return NULL;
 
@@ -820,8 +823,8 @@ librdf_hash_get_all_iterator_next_method(void* iterator)
 static void*
 librdf_hash_get_all_iterator_get_method(void* iterator, int flags) 
 {
-  librdf_hash_get_all_iterator_context* context=(librdf_hash_get_all_iterator_context*)iterator;
-  void *result=NULL;
+  librdf_hash_get_all_iterator_context* context = (librdf_hash_get_all_iterator_context*)iterator;
+  void *result = NULL;
   
   if(context->is_end)
     return NULL;
@@ -833,22 +836,22 @@ librdf_hash_get_all_iterator_get_method(void* iterator, int flags)
        * might as well return something useful to signify not-end-of-list.
        */
 
-      result=&context;
+      result = context;
       break;
 
     case LIBRDF_ITERATOR_GET_METHOD_GET_KEY:
-      result=&context->next_key;
+      result = &context->next_key;
       break;
       
     case LIBRDF_ITERATOR_GET_METHOD_GET_VALUE:
-      result=&context->next_value;
+      result = &context->next_value;
       break;
 
     default:
       librdf_log(context->hash->world, 
                  0, LIBRDF_LOG_ERROR, LIBRDF_FROM_HASH, NULL,
                  "Unknown iterator method flag %d", flags);
-      result=NULL;
+      result = NULL;
       break;
   }
 
@@ -1009,7 +1012,8 @@ librdf_hash_keys(librdf_hash* hash, librdf_hash_datum *key)
   int status;
   librdf_iterator* iterator;
   
-  context=(librdf_hash_keys_iterator_context*)LIBRDF_CALLOC(librdf_hash_keys_iterator_context, 1, sizeof(librdf_hash_keys_iterator_context));
+  context = LIBRDF_CALLOC(librdf_hash_keys_iterator_context*, 1,
+                          sizeof(*context));
   if(!context)
     return NULL;
 
@@ -1080,9 +1084,9 @@ librdf_hash_keys_iterator_next_method(void* iterator)
 static void*
 librdf_hash_keys_iterator_get_method(void* iterator, int flags) 
 {
-  librdf_hash_keys_iterator_context* context=(librdf_hash_keys_iterator_context*)iterator;
-  void *result=NULL;
-  
+  librdf_hash_keys_iterator_context* context = (librdf_hash_keys_iterator_context*)iterator;
+  void *result = NULL;
+
   if(context->is_end)
     return NULL;
 
@@ -1093,15 +1097,15 @@ librdf_hash_keys_iterator_get_method(void* iterator, int flags)
        * might as well return something useful to signify not-end-of-list.
        */
 
-      result=&context;
+      result = context;
       break;
       
     case LIBRDF_ITERATOR_GET_METHOD_GET_KEY:
-      result=&context->next_key;
+      result = &context->next_key;
       break;
       
     default:
-      result=NULL;
+      result = NULL;
   }
 
   return result;
@@ -1174,6 +1178,8 @@ librdf_hash_print(librdf_hash* hash, FILE *fh)
 
   key=librdf_new_hash_datum(hash->world, NULL, 0);
   value=librdf_new_hash_datum(hash->world, NULL, 0);
+  if(!key || !value)
+    return;
 
   iterator=librdf_hash_get_all(hash, key, value);
   while(!librdf_iterator_end(iterator)) {
@@ -1184,12 +1190,12 @@ librdf_hash_print(librdf_hash* hash, FILE *fh)
     v=(librdf_hash_datum *)librdf_iterator_get_value(iterator);
     
     fputs("  '", fh);
-    l=fwrite(k->data, k->size, 1, fh);
+    l=fwrite(k->data, 1, k->size, fh);
     if(l != k->size)
       break;
     
     fputs("'=>'", fh);
-    l=fwrite(v->data, v->size, 1, fh);
+    l=fwrite(v->data, 1, v->size, fh);
     if(l != v->size)
       break;
 
@@ -1224,6 +1230,8 @@ librdf_hash_print_keys(librdf_hash* hash, FILE *fh)
   fputs("{\n", fh);
 
   key=librdf_new_hash_datum(hash->world, NULL, 0);
+  if(!key)
+    return;
 
   iterator=librdf_hash_keys(hash, key);
   while(!librdf_iterator_end(iterator)) {
@@ -1231,7 +1239,7 @@ librdf_hash_print_keys(librdf_hash* hash, FILE *fh)
     size_t l;
     
     fputs("  '", fh);
-    l=fwrite(k->data, k->size, 1, fh);
+    l=fwrite(k->data, 1, k->size, fh);
     if(l != k->size)
       break;
     fputs("'\n", fh);
@@ -1284,7 +1292,7 @@ librdf_hash_print_values(librdf_hash* hash, const char *key_string, FILE *fh)
       fputs(", ", fh);
       
     fputc('\'', fh);
-    l=fwrite(v->data, v->size, 1, fh);
+    l=fwrite(v->data, 1, v->size, fh);
     if(l != v->size)
       break;
 
@@ -1353,8 +1361,7 @@ librdf_hash_from_string(librdf_hash* hash, const char *string)
 
   p=string;
   key=NULL; key_len=0;
-  value=NULL; value_len=0;
-  backslashes=0;
+  value=NULL;
   state=HFS_PARSE_STATE_INIT;
   while(*p) {
 
@@ -1433,7 +1440,7 @@ librdf_hash_from_string(librdf_hash* hash, const char *string)
           break;
         }
         p++;
-        state=HFS_PARSE_STATE_VALUE;
+        /* state=HFS_PARSE_STATE_VALUE; */
         /* fall through to next state */
         
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
@@ -1464,7 +1471,7 @@ librdf_hash_from_string(librdf_hash* hash, const char *string)
         /* ' at end of value found */
         value_len=p-value;
         real_value_len=value_len-backslashes;
-        new_value=(char*)LIBRDF_MALLOC(cstring, real_value_len+1);
+        new_value = LIBRDF_MALLOC(char*, real_value_len + 1);
         if(!new_value)
           return 1;
         for(i=0, to=new_value; i<(int)value_len; i++){
@@ -1483,7 +1490,7 @@ librdf_hash_from_string(librdf_hash* hash, const char *string)
         
         librdf_hash_put(hash, &hd_key, &hd_value);
         
-        LIBRDF_FREE(cstring, new_value);
+        LIBRDF_FREE(char*, new_value);
         
 #if defined(LIBRDF_DEBUG) && LIBRDF_DEBUG > 1
         LIBRDF_DEBUG1("after decoding ");
@@ -1537,47 +1544,160 @@ librdf_hash_from_array_of_strings(librdf_hash* hash, const char **array)
 }
 
 
+/**
+ * librdf_hash_to_string:
+ * @hash: #librdf_hash object
+ * @filter: NULL terminated list of keys to ignore
+ *
+ * Format the hash as a string, suitable for parsing by librdf_hash_from_string.
+ *
+ * Note: this method allocates a new string since this is a _to_ method
+ * and the caller must call librdf_free_memory() to free the resulting memory.
+ *
+ * Return value: string representation of the hash or NULL on failure
+ **/
+char*
+librdf_hash_to_string(librdf_hash* hash, const char *filter[])
+{
+  librdf_hash_datum *key = NULL, *value = NULL;
+  librdf_iterator* iterator = NULL;
+  raptor_stringbuffer* sb;
+  char* result = NULL;
+  size_t len;
+
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(hash, librdf_hash, NULL);
+
+  sb = raptor_new_stringbuffer();
+  if(!sb)
+    goto tidy;
+
+  key=librdf_new_hash_datum(hash->world, NULL, 0);
+  value=librdf_new_hash_datum(hash->world, NULL, 0);
+  if (!key || !value)
+    goto tidy;
+
+  iterator=librdf_hash_get_all(hash, key, value);
+  if (!iterator)
+    goto tidy;
+
+  while(!librdf_iterator_end(iterator)) {
+    librdf_hash_datum *k, *v;
+    int key_is_filtered = 0;
+    size_t i;
+
+    k=(librdf_hash_datum *)librdf_iterator_get_key(iterator);
+    v=(librdf_hash_datum *)librdf_iterator_get_value(iterator);
+    if (!k || !v)
+      break;
+
+    /* Is this one of the keys that we are ignoring? */
+    if(filter) {
+      for(i=0; filter[i]; i++) {
+        size_t f_len = strlen(filter[i]);
+        if(f_len == k->size && 
+           strncmp((const char*)k->data, filter[i], f_len) == 0) {
+          key_is_filtered = 1;
+          break;
+        }
+      }
+    }
+
+    if(!key_is_filtered) {
+      if(raptor_stringbuffer_length(sb) > 0)
+        raptor_stringbuffer_append_counted_string(sb, (unsigned char*)", ", 2, 1);
+
+      raptor_stringbuffer_append_counted_string(sb, (unsigned char*)k->data, k->size, 1);
+      raptor_stringbuffer_append_counted_string(sb, (unsigned char*)"='", 2, 1);
+
+      for(i=0; i < v->size; i++) {
+        char c = ((char*)v->data)[i];
+        switch (c) {
+          case '\'':
+            raptor_stringbuffer_append_counted_string(sb, (unsigned char*)"\\'", 2, 1);
+            break;
+          case '\\':
+            raptor_stringbuffer_append_counted_string(sb, (unsigned char*)"\\\\", 2, 1);
+            break;
+          default:
+            raptor_stringbuffer_append_counted_string(sb, (unsigned char*)&c, 1, 1);
+            break;
+        }
+      }
+
+      raptor_stringbuffer_append_counted_string(sb, (unsigned char*)"'", 1, 1);
+    }
+
+    librdf_iterator_next(iterator);
+  }
+
+  /* Generate a string result */
+  len=raptor_stringbuffer_length(sb);
+  result = (char *)librdf_alloc_memory(len + 1);
+  if (result)
+    raptor_stringbuffer_copy_to_string(sb, (unsigned char*)result, len);
+
+
+  tidy:
+  if(iterator)
+    librdf_free_iterator(iterator);
+  if(value)
+    librdf_free_hash_datum(value);
+  if(key)
+    librdf_free_hash_datum(key);
+  if(sb)
+    raptor_free_stringbuffer(sb);
+
+  return result;
+}
+
+
 /**
  * librdf_hash_get_as_boolean:
  * @hash: #librdf_hash object
  * @key: key string to look up
  *
  * Lookup a hash key and decode value as a boolean.
+ *
+ * False values: "no", "false"
+ * True values: "yes", "true"
  * 
  * Return value: >0 (for true), 0 (for false) or <0 (for key not found or not known boolean value)
  **/
 int
-librdf_hash_get_as_boolean (librdf_hash* hash, const char *key) 
+librdf_hash_get_as_boolean(librdf_hash* hash, const char *key) 
 {
-  int bvalue= (-1);
+  int bvalue = (-1);
   char *value;
 
-  value=librdf_hash_get(hash, key);
+  value = librdf_hash_get(hash, key);
   if(!value)
     /* does not exist - fail */
     return -1;
 
   switch(strlen(value)) {
   case 2: /* try 'no' */
-    if(*value=='n' && value[1]=='o')
-      bvalue=0;
+    if(*value == 'n' && value[1] == 'o')
+      bvalue = 0;
     break;
+
   case 3: /* try 'yes' */
-    if(*value=='y' && value[1]=='e' && value[2]=='s')
-      bvalue=1;
+    if(*value == 'y' && value[1] == 'e' && value[2] == 's')
+      bvalue = 1;
     break;
+
   case 4: /* try 'true' */
-    if(*value=='t' && value[1]=='r' && value[2]=='u' && value[3]=='e')
-      bvalue=1;
+    if(*value == 't' && value[1] == 'r' && value[2] == 'u' && value[3] == 'e')
+      bvalue = 1;
     break;
+
   case 5: /* try 'false' */
     if(!strncmp(value, "false", 5))
-      bvalue=1;
+      bvalue = 0;
     break;
   /* no need for default, bvalue is set above */
   }
 
-  LIBRDF_FREE(cstring, value);
+  LIBRDF_FREE(char*, value);
 
   return bvalue;
 }
@@ -1611,10 +1731,11 @@ librdf_hash_get_as_long (librdf_hash* hash, const char *key)
   if(end_ptr == value)
     lvalue= (-1);
 
-  LIBRDF_FREE(cstring, value);
+  LIBRDF_FREE(char*, value);
   return lvalue;
 }
 
+
 /**
  * librdf_hash_put_strings:
  * @hash: hash object
@@ -1728,7 +1849,7 @@ librdf_hash_interpret_template(const unsigned char* template_string,
   /* Generate a string result */
   len=raptor_stringbuffer_length(sb);
   if(len) {
-    result=(unsigned char*)LIBRDF_MALLOC(cstring, len+1);
+    result = LIBRDF_MALLOC(unsigned char*, len + 1);
     raptor_stringbuffer_copy_to_string(sb, result, len);
   }
   
@@ -1775,12 +1896,14 @@ main(int argc, char *argv[])
   const char *test_hash_delete_key="size";
   const unsigned char* template_string=(const unsigned char*)"the shape is %{shape} and the sides are %{sides} created by %{rubik}";
   const unsigned char* template_expected=(const unsigned char*)"the shape is cube and the sides are 6 created by ";
+  const char * filter_string[] = {"field1", NULL};
   int i,j;
   const char *type;
   librdf_hash_datum hd_key, hd_value; /* on stack */
   const char *program=librdf_basename((const char*)argv[0]);
   int b;
   long l;
+  char* string_result;
   unsigned char* template_result;
   librdf_world *world;
   
@@ -1910,7 +2033,7 @@ main(int argc, char *argv[])
       librdf_hash_datum *k=(librdf_hash_datum*)librdf_iterator_get_key(iterator);
       char *key_string;
       
-      key_string=(char*)LIBRDF_MALLOC(cstring, k->size+1);
+      key_string = LIBRDF_MALLOC(char*, k->size + 1);
       if(!key_string)
         break;
       strncpy(key_string, (char*)k->data, k->size);
@@ -1926,7 +2049,7 @@ main(int argc, char *argv[])
       l=librdf_hash_get_as_long(h2, key_string);
       fprintf(stdout, "%ld (decimal, -1 Bad)\n", l);
       
-      LIBRDF_FREE(cstring, key_string);
+      LIBRDF_FREE(char*, key_string);
       librdf_iterator_next(iterator);
     }
     if(iterator)
@@ -1935,14 +2058,14 @@ main(int argc, char *argv[])
     librdf_free_hash_datum(key_hd);
   }
 
-
   fprintf(stdout, "%s: Freeing hash\n", program);
   /* close() done automatically by free so not required */
   /* librdf_hash_close(h2); */
   librdf_free_hash(h2);
 
+
   h2=librdf_new_hash(world, NULL);
-  fprintf(stdout, "%s: Initialising hash from string >>%s<<\n", program, 
+  fprintf(stdout, "%s: Initialising hash from string >>%s<<\n", program,
           test_hash_string);
   librdf_hash_from_string (h2, test_hash_string);
   fprintf(stdout, "%s: resulting ", program);
@@ -1950,9 +2073,50 @@ main(int argc, char *argv[])
   fputc('\n', stdout);
   fprintf(stdout, "%s: values count %d\n", program, librdf_hash_values_count(h2));
 
+  fprintf(stdout, "%s: Converting hash back to a string\n", program);
+  string_result=librdf_hash_to_string(h2, NULL);
+
+  /* Order is not guaranteed, so sadly we can't just do a full string comparison */
+  if(!strstr(string_result, "field1='value1'")) {
+    fprintf(stdout, "%s: Did not see field1='value1' in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else if(!strstr(string_result, "field2='\\'value2'")) {
+    fprintf(stdout, "%s: Did not see field2='\\'value2'' in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else if(!strstr(string_result, "field3='\\\\'")) {
+    fprintf(stdout, "%s: Did not see field3='\\\\' in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else if(!strstr(string_result, "field4='\\\\\\'")) {
+    fprintf(stdout, "%s: Did not see field4='\\\\\\' in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else if(!strstr(string_result, "field5='a'")) {
+    fprintf(stdout, "%s: Did not see field5='a' in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else {
+    fprintf(stdout, "%s: resulting in >>%s<<\n", program, string_result);
+  }
+  librdf_free_memory(string_result);
+
+  fprintf(stdout, "%s: Converting hash back to a string with filter\n", program);
+  string_result=librdf_hash_to_string(h2, filter_string);
+  if(strstr(string_result, "field1")) {
+    fprintf(stdout, "%s: Was not expecting >>field1<< to be in the generated string >>%s<<\n",
+            program, string_result);
+    exit(1);
+  } else {
+    fprintf(stdout, "%s: resulting in >>%s<<\n", program, string_result);
+  }
+  librdf_free_memory(string_result);
+
   librdf_free_hash(h2);
 
-   
+
+
   fprintf(stdout, "%s: Subtituting into template >>%s<<\n", program, 
           template_string);
   h2=librdf_new_hash(world, NULL);
@@ -1968,7 +2132,7 @@ main(int argc, char *argv[])
   } else
     fprintf(stdout, "%s: resulting in >>%s<<\n", program, template_result);
 
-  LIBRDF_FREE(cstring, template_result);
+  LIBRDF_FREE(char*, template_result);
 
   librdf_free_hash(h2);
 
