@@ -73,11 +73,17 @@ librdf_finish_model(librdf_world *world)
 }
 
 
-/*
- * librdf_model_supports_contexts - helper function to determine if this model supports contexts
+/**
+ * librdf_model_supports_contexts:
+ * @model: the model object
+ *
+ * Check if this model supports contexts
+ *
+ * Return value: non-0 if contexts are supported
  **/
-static REDLAND_INLINE int
-librdf_model_supports_contexts(librdf_model* model) {
+int
+librdf_model_supports_contexts(librdf_model* model)
+{
   return model->supports_contexts;
 }
 
@@ -122,7 +128,8 @@ librdf_model_register_factory(librdf_world *world,
 #endif
 
   if(!world->models) {
-    world->models=raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_model_factory, NULL);
+    world->models = raptor_new_sequence((raptor_data_free_handler)librdf_free_model_factory, NULL);
+
     if(!world->models)
       goto oom;
   }
@@ -138,17 +145,16 @@ librdf_model_register_factory(librdf_world *world,
     }
   }
 
-  model=(librdf_model_factory*)LIBRDF_CALLOC(librdf_model_factory, 1,
-                                             sizeof(librdf_model_factory));
+  model = LIBRDF_CALLOC(librdf_model_factory*, 1, sizeof(*model));
   if(!model)
     goto oom;
 
-  model->name=(char*)LIBRDF_MALLOC(cstring, strlen(name)+1);
+  model->name = LIBRDF_MALLOC(char*, strlen(name) + 1);
   if(!model->name)
     goto oom_tidy;
   strcpy(model->name, name);
 
-  model->label=(char*)LIBRDF_MALLOC(cstring, strlen(label)+1);
+  model->label = LIBRDF_MALLOC(char*, strlen(label) + 1);
   if(!model->label)
     goto oom_tidy;
   strcpy(model->label, label);
@@ -315,7 +321,7 @@ librdf_new_model_with_options(librdf_world *world,
   if(!storage)
     return NULL;
   
-  model=(librdf_model*)LIBRDF_CALLOC(librdf_model, 1, sizeof(librdf_model));
+  model = LIBRDF_CALLOC(librdf_model*, 1, sizeof(*model));
   if(!model)
     return NULL;
   
@@ -327,7 +333,7 @@ librdf_new_model_with_options(librdf_world *world,
     return NULL;
   }
     
-  model->context=LIBRDF_CALLOC(data, 1, model->factory->context_length);
+  model->context = LIBRDF_CALLOC(void*, 1, model->factory->context_length);
 
   if(!model->context || model->factory->create(model, storage, options)) {
     if(model->context)
@@ -392,6 +398,9 @@ librdf_free_model(librdf_model *model)
   librdf_iterator* iterator;
   librdf_model* m;
 
+  if(!model)
+    return;
+  
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(model, librdf_model);
 
   if(--model->usage)
@@ -1126,6 +1135,7 @@ librdf_model_has_arc_out(librdf_model *model, librdf_node *node,
 
 
 
+#ifndef REDLAND_DISABLE_DEPRECATED
 /**
  * librdf_model_print:
  * @model: the model object
@@ -1135,22 +1145,72 @@ librdf_model_has_arc_out(librdf_model *model, librdf_node *node,
  * 
  * This method is for debugging and the format of the output should
  * not be relied on.
+ *
+ * @Deprecated: Use librdf_model_write() to write to #raptor_iostream
+ * which can be made to write to a string.  Use a #librdf_serializer
+ * to write proper syntax formats.
+ *
  **/
 void
 librdf_model_print(librdf_model *model, FILE *fh)
 {
-  librdf_stream* stream;
+  raptor_iostream *iostr;
   
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(model, librdf_model);
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN(fh, FILE*);
 
-  stream=librdf_model_as_stream(model);
-  if(!stream)
+  iostr = raptor_new_iostream_to_file_handle(model->world->raptor_world_ptr, fh);
+  if(!iostr)
     return;
-  fputs("[[\n", fh);
-  librdf_stream_print(stream, fh);
-  fputs("]]\n", fh);
-  librdf_free_stream(stream);
+  
+  librdf_model_write(model, iostr);
+  
+  raptor_free_iostream(iostr);
+}
+#endif
+
+
+/**
+ * librdf_model_write:
+ * @model: the model object
+ * @iostr: the iostream to write to
+ *
+ * Write a model to an iostream in a debug format.
+ * 
+ * This method is for debugging and the format of the output should
+ * not be relied on.  In particular, when contexts are used the
+ * result may be 4 nodes.
+ *
+ * Return value: non-0 on failure
+ **/
+int
+librdf_model_write(librdf_model *model, raptor_iostream* iostr)
+{
+  int rc = 1;
+  librdf_stream* stream;
+  
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(model, librdf_model, 1);
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(iostr, raptor_iostream, 1);
+
+  stream = librdf_model_as_stream(model);
+  if(!stream)
+    goto tidy;
+  
+  if(raptor_iostream_counted_string_write("[[\n", 3, iostr))
+    goto tidy;
+  if(librdf_stream_write(stream, iostr))
+    goto tidy;
+  if(raptor_iostream_counted_string_write("]]\n", 3, iostr))
+    goto tidy;
+  
+  /* success */
+  rc = 0;
+
+  tidy:
+  if(stream)
+    librdf_free_stream(stream);
+
+  return rc;
 }
 
 
@@ -1314,7 +1374,7 @@ librdf_model_context_remove_statements(librdf_model* model,
     librdf_model_context_remove_statement(model, context, statement);
     librdf_stream_next(stream);
   }
-  librdf_free_stream(stream);  
+  librdf_free_stream(stream);
   return 0;
 }
 
@@ -1633,14 +1693,10 @@ librdf_model_load(librdf_model* model, librdf_uri *uri,
     mime_type=NULL;
 
   if(!name)
-#ifdef RAPTOR_V2_AVAILABLE
-    name = raptor_guess_parser_name_v2(model->world->raptor_world_ptr,
-                                       (raptor_uri*)type_uri, mime_type,
-                                       NULL, 0, librdf_uri_as_string(uri));
-#else
-    name = raptor_guess_parser_name((raptor_uri*)type_uri, mime_type,
-                                    NULL, 0, librdf_uri_as_string(uri));
-#endif
+    name = raptor_world_guess_parser_name(model->world->raptor_world_ptr,
+                                          (raptor_uri*)type_uri, mime_type,
+                                          NULL, 0, librdf_uri_as_string(uri));
+
   parser=librdf_new_parser(model->world, name, NULL, NULL);
   if(!parser)
     return 1;
@@ -1664,7 +1720,7 @@ librdf_model_load(librdf_model* model, librdf_uri *uri,
  *
  * If the name field is NULL, the default serializer will be used.
  *
- * Note: the returned string must be freed by the caller.
+ * Note: the returned string must be freed by the caller using librdf_free_memory().
  *
  * Return value: new string or NULL on failure
  **/
@@ -1869,9 +1925,84 @@ int main(int argc, char *argv[]);
 "</rdf:RDF>"
 
 int test_model_cloning(char const *program, librdf_world *);
+int test_model(librdf_world *world, const char *program,
+    const char *storage_type, const char *storage_name, const char* storage_options);
 
 int
 main(int argc, char *argv[]) 
+{
+  const char *program=librdf_basename((const char*)argv[0]);
+  /* initialise dependent modules - all of them! */
+  librdf_world *world=librdf_new_world();
+  const char* storage_type;
+  const char* storage_name;
+  const char* storage_options;
+  int i;
+  int status=0;
+
+  librdf_world_open(world);
+
+  /* Test model cloning first */
+  if(test_model_cloning(program, world)) {
+    status = 1;
+    goto tidy;
+  }
+
+  /* Get storage configuration */
+  storage_type=getenv("REDLAND_TEST_STORAGE_TYPE");
+  storage_name=getenv("REDLAND_TEST_STORAGE_NAME");
+  storage_options=getenv("REDLAND_TEST_STORAGE_OPTIONS");
+  if(!(storage_type && storage_name && storage_options)) {
+    /* test all storages */
+    const char* const storages[] = {
+      "memory", NULL, "write='yes',new='yes',contexts='yes'",
+#ifdef STORAGE_HASHES
+#ifdef HAVE_BDB_HASH
+      "hashes", "test", "hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'",
+#else
+      "hashes", "test", "hash-type='memory',write='yes',new='yes',contexts='yes'",
+#endif
+#endif
+#ifdef STORAGE_TREES
+      "trees", "test", "contexts='yes'",
+#endif
+#ifdef STORAGE_FILE
+      "file", "test.rdf", NULL,
+#endif
+#ifdef STORAGE_MYSQL
+      "mysql", "test", "host='localhost',database='test'",
+#endif
+#ifdef STORAGE_POSTGRESQL
+      "postgresql", "test", "host='localhost',database='test'",
+#endif
+#ifdef STORAGE_TSTORE
+      "tstore", "test", "host='localhost',database='test'",
+#endif
+#ifdef STORAGE_SQLITE
+      "sqlite", "test", "new='yes'",
+#endif
+       NULL, NULL, NULL
+    };
+
+    for (i=0; storages[i]; i += 3) {
+      if (test_model(world, program, storages[i], storages[i+1], storages[i+2])) {
+        status = 1;
+        break;
+      }
+    }
+  } else {
+    status = test_model(world, program, storage_type, storage_name, storage_options);
+  }
+
+  tidy:
+  librdf_free_world(world);
+
+  return status;
+}
+
+int
+test_model(librdf_world *world, const char *program,
+    const char *storage_type, const char *storage_name, const char *storage_options)
 {
   librdf_storage* storage;
   librdf_model* model;
@@ -1885,45 +2016,28 @@ main(int argc, char *argv[])
   librdf_uri* uris[URI_STRING_COUNT];
   librdf_node* nodes[URI_STRING_COUNT];
   int i;
-  const char *program=librdf_basename((const char*)argv[0]);
-  /* initialise dependent modules - all of them! */
-  librdf_world *world=librdf_new_world();
-  librdf_iterator *iterator, *second_iterator;
+  librdf_iterator *iterator;
   librdf_node *n1, *n2;
   int count;
   int expected_count;
-#define EXPECTED_BAD_STRING_LENGTH 317
+#define EXPECTED_BAD_STRING_LENGTH 1139
   librdf_uri* base_uri;
   unsigned char *string;
   size_t string_length=0;
   int remove_count=0;
   int status=0;
-  const char* storage_type;
-  const char* storage_name;
-  const char* storage_options;
+  raptor_iostream* iostr;
+  librdf_node* literal_node;
+  char literal[6];
 
-  librdf_world_open(world);
-
-  /* Test model cloning first */
-  if(test_model_cloning(program, world))
-    return(1);
-  
-  /* Get storage configuration */
-  storage_type=getenv("REDLAND_TEST_STORAGE_TYPE");
-  storage_name=getenv("REDLAND_TEST_STORAGE_NAME");
-  storage_options=getenv("REDLAND_TEST_STORAGE_OPTIONS");
-  if(!(storage_type && storage_name && storage_options)) {
-    /* default is to test in memory */
-    storage_type="memory";
-    storage_name=NULL;
-    storage_options="write='yes',new='yes',contexts='yes'";
-  }
+  iostr = raptor_new_iostream_to_file_handle(world->raptor_world_ptr, stderr);
 
   fprintf(stderr, "%s: Creating new %s storage\n", program, storage_type);
   storage=librdf_new_storage(world, storage_type, storage_name, storage_options);
   if(!storage) {
-    fprintf(stderr, "%s: Failed to create new %s storage name %s with options %s\n", program, storage_type, storage_name, storage_options);
-    return(1);
+    fprintf(stderr, "%s: WARNING: Failed to create new %s storage name %s with options %s\n", program, storage_type, storage_name, storage_options);
+    raptor_free_iostream(iostr);
+    return(0);
   }
 
   fprintf(stderr, "%s: Creating model\n", program);
@@ -1961,8 +2075,98 @@ main(int argc, char *argv[])
   librdf_free_statement(statement);
 
   fprintf(stderr, "%s: Printing model\n", program);
-  librdf_model_print(model, stderr);
+  librdf_model_write(model, iostr);
   
+#define TEST_SIMILAR_COUNT 9
+
+  /* add some similar statements */
+  fprintf(stderr, "%s: Adding %d similar statements\n", program, TEST_SIMILAR_COUNT);
+  for(i=0; i < TEST_SIMILAR_COUNT; i++) {
+    char literal[6];
+    strncpy(literal, "DaveX", 6);
+    literal[4]='0'+i;
+    statement=librdf_new_statement(world);
+    librdf_statement_set_subject(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/"));
+    librdf_statement_set_predicate(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator"));
+    librdf_statement_set_object(statement, librdf_new_node_from_literal(world, (const unsigned char*)literal, NULL, 0));
+
+    librdf_model_add_statement(model, statement);
+    librdf_free_statement(statement);
+  }
+
+  /* targets */
+  fprintf(stderr, "%s: iterating similar statements\n", program);
+  n1=librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/");
+  n2=librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator");
+  iterator=librdf_model_get_targets(model, n1, n2);
+  if(!iterator) {
+    fprintf(stderr, "%s: librdf_model_get_targets failed\n", program);
+    status=1;
+  }
+
+  fprintf(stderr, "%s: iterating similar statements again\n", program);
+  for(count=0; !librdf_iterator_end(iterator); librdf_iterator_next(iterator), count++) {
+    librdf_node* n=(librdf_node*)librdf_iterator_get_object(iterator);
+    fputs("  ", stderr);
+    librdf_node_print(n, stderr);
+    fputs("\n", stderr);
+  }
+  librdf_free_iterator(iterator);
+
+  /* delete first, last, and another statement */
+  statement=librdf_new_statement(world);
+  librdf_statement_set_subject(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/"));
+  librdf_statement_set_predicate(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator"));
+
+  strncpy(literal, "DaveX", 6);
+
+  literal[4]='0';
+  fprintf(stderr, "%s: Removing statement with literal '%s'\n", program, literal);
+  literal_node = librdf_new_node_from_literal(world, (const unsigned char*)literal, NULL, 0);
+  librdf_statement_set_object(statement, literal_node);
+  librdf_model_remove_statement(model, statement);
+  librdf_statement_set_object(statement, NULL);
+  librdf_free_node(literal_node);
+  
+  literal[4]='0' + (TEST_SIMILAR_COUNT / 2);
+  fprintf(stderr, "%s: Removing statement with literal '%s'\n", program, literal);
+  literal_node = librdf_new_node_from_literal(world, (const unsigned char*)literal, NULL, 0);
+  librdf_statement_set_object(statement, literal_node);
+  librdf_model_remove_statement(model, statement);
+  librdf_statement_set_object(statement, NULL);
+  librdf_free_node(literal_node);
+
+  literal[4]='0' + (TEST_SIMILAR_COUNT - 1);
+  fprintf(stderr, "%s: Removing statement with literal '%s'\n", program, literal);
+  literal_node = librdf_new_node_from_literal(world, (const unsigned char*)literal, NULL, 0);
+  librdf_statement_set_object(statement, literal_node);
+  librdf_model_remove_statement(model, statement);
+  librdf_statement_set_object(statement, NULL);
+  librdf_free_node(literal_node);
+
+  librdf_free_statement(statement);
+
+  iterator=librdf_model_get_targets(model, n1, n2);
+  fprintf(stderr, "%s: iterating similar statements again\n", program);
+  for(count=0; !librdf_iterator_end(iterator); librdf_iterator_next(iterator), count++) {
+    librdf_node* n=(librdf_node*)librdf_iterator_get_object(iterator);
+    fputs("  ", stderr);
+    librdf_node_print(n, stderr);
+    fputs("\n", stderr);
+  }
+  librdf_free_iterator(iterator);
+  expected_count=TEST_SIMILAR_COUNT - 3;
+  if(count != expected_count) {
+    fprintf(stderr, "%s: model has %d similar statements, expected %d\n", program, count, expected_count);
+    status=1;
+  }
+
+  librdf_free_node(n1);
+  librdf_free_node(n2);
+
+  if (!model->supports_contexts)
+    goto done;
+
   parser=librdf_new_parser(world, parser_name, NULL, NULL);
   if(!parser) {
     fprintf(stderr, "%s: Failed to create new parser type %s\n", program,
@@ -1980,13 +2184,13 @@ main(int argc, char *argv[])
                                                      file_content[i], uris[i]))) {
       fprintf(stderr, "%s: Failed to parse RDF from %s as stream\n", program,
               librdf_uri_as_string(uris[i]));
-      exit(1);
+      return(1);
     }
     librdf_model_context_add_statements(model, nodes[i], stream);
     librdf_free_stream(stream);
 
     fprintf(stderr, "%s: Printing model\n", program);
-    librdf_model_print(model, stderr);
+    librdf_model_write(model, iostr);
   }
 
 
@@ -2028,7 +2232,7 @@ main(int argc, char *argv[])
   }
   librdf_free_node(n1);
   librdf_free_node(n2);
-  
+
 
   /* targets */
   n1=librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/net/dajobe/");
@@ -2131,7 +2335,7 @@ main(int argc, char *argv[])
     librdf_model_context_remove_statements(model, nodes[i]);
 
     fprintf(stderr, "%s: Printing model\n", program);
-    librdf_model_print(model, stderr);
+    librdf_model_write(model, iostr);
   }
 
   fprintf(stderr, "%s: Serializing model\n", program);
@@ -2146,119 +2350,37 @@ main(int argc, char *argv[])
     return 1;
   }
   librdf_free_uri(base_uri);
-  free(string);
+  librdf_free_memory(string);
   fprintf(stderr, "%s: Serialized OK\n", program);
 
-
-#define TEST_SIMILAR_COUNT 9
-
-  /* add some similar statements */
-  fprintf(stderr, "%s: Adding %d similar statements\n", program, TEST_SIMILAR_COUNT);
-  for(i=0; i < TEST_SIMILAR_COUNT; i++) {
-    char literal[6];
-    strncpy(literal, "DaveX", 6);
-    literal[4]='0'+i;
-    statement=librdf_new_statement(world);
-    librdf_statement_set_subject(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/"));
-    librdf_statement_set_predicate(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator"));
-    librdf_statement_set_object(statement, librdf_new_node_from_literal(world, (const unsigned char*)literal, NULL, 0));
-    
-    librdf_model_add_statement(model, statement);
-    librdf_free_statement(statement);
-  }
-
-  /* targets */
-  fprintf(stderr, "%s: iterating similar statements\n", program);
-  n1=librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/");
-  n2=librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator");
-  iterator=librdf_model_get_targets(model, n1, n2);
-  if(!iterator) {
-    fprintf(stderr, "%s: librdf_model_get_targets failed\n", program);
-    status=1;
-  }
-
-  fprintf(stderr, "%s: iterating similar statements again\n", program);
-  second_iterator=librdf_model_get_targets(model, n1, n2);
-
-  /* Deleting 2 statements but only 1 before counting */
-  expected_count= TEST_SIMILAR_COUNT - 1;
-  for(count=0;
-      !librdf_iterator_end(iterator); 
-      librdf_iterator_next(iterator), librdf_iterator_next(second_iterator), count++) {
-    librdf_node* n;
-    char literal[6];
-
-    if(count == 2 ) {
-      strncpy(literal, "DaveX", 6);
-      literal[4]='0'+count;
-      statement=librdf_new_statement(world);
-      librdf_statement_set_subject(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/"));
-      librdf_statement_set_predicate(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator"));
-      librdf_statement_set_object(statement, librdf_new_node_from_literal(world, (const unsigned char*)literal, NULL, 0));
-
-      fprintf(stderr, "%s: Removing statement with literal '%s'\n",
-              program, literal);
-      
-      librdf_model_remove_statement(model, statement);
-      librdf_free_statement(statement);
-      remove_count++;
-    }
-  
-    n=(librdf_node*)librdf_iterator_get_object(iterator);
-    fputs("  ", stderr);
-    librdf_node_print(n, stderr);
-    fputs("\n", stderr);
-
-    if(count == 5 ) {
-      strncpy(literal, "DaveX", 6);
-      literal[4]='0'+count+remove_count;
-      statement=librdf_new_statement(world);
-      librdf_statement_set_subject(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://example.org/"));
-      librdf_statement_set_predicate(statement, librdf_new_node_from_uri_string(world, (const unsigned char*)"http://purl.org/dc/elements/1.1/creator"));
-      librdf_statement_set_object(statement, librdf_new_node_from_literal(world, (const unsigned char*)literal, NULL, 0));
-
-      fprintf(stderr, "%s: Removing statement with literal '%s'\n",
-              program, literal);
-      
-      librdf_model_remove_statement(model, statement);
-      librdf_free_statement(statement);
-      remove_count++;
-    }
-  
-  }
-  librdf_free_iterator(iterator);
-  if(count != expected_count) {
-    fprintf(stderr, "%s: librdf_model_get_targets returned %d nodes, expected %d\n", program, count, expected_count);
-    status=1;
-  }
-  librdf_free_node(n1);
-  librdf_free_node(n2);
-  
-
-  librdf_free_iterator(second_iterator);
-
   fprintf(stderr, "%s: Freeing URIs and Nodes\n", program);
   for (i=0; i<URI_STRING_COUNT; i++) {
     librdf_free_uri(uris[i]);
     librdf_free_node(nodes[i]);
   }
   
+done:
   fprintf(stderr, "%s: Freeing model\n", program);
   librdf_free_model(model);
 
   fprintf(stderr, "%s: Freeing storage\n", program);
   librdf_free_storage(storage);
 
-  librdf_free_world(world);
-  
+  raptor_free_iostream(iostr);
+
   return status;
 }
 
-int test_model_cloning(char const *program, librdf_world *world) {
-  librdf_storage *storage;
-  librdf_model *model1;
-  librdf_model *model2;
-  librdf_model *model3;
+int
+test_model_cloning(char const *program, librdf_world *world)
+{
+  int status = 0;
+  librdf_storage *storage1 = NULL;
+  librdf_storage *storage2 = NULL;
+  librdf_storage *storage3 = NULL;
+  librdf_model *model1 = NULL;
+  librdf_model *model2 = NULL;
+  librdf_model *model3 = NULL;
   const char* storage_type;
   const char* storage_name;
   const char* storage_options;
@@ -2266,59 +2388,75 @@ int test_model_cloning(char const *program, librdf_world *world) {
   fprintf(stderr, "%s: Testing model cloning\n", program);
 
   /* Get storage configuration */
-  storage_type=getenv("REDLAND_TEST_CLONING_STORAGE_TYPE");
-  storage_name=getenv("REDLAND_TEST_CLONING_STORAGE_NAME");
-  storage_options=getenv("REDLAND_TEST_CLONING_STORAGE_OPTIONS");
+  storage_type = getenv("REDLAND_TEST_CLONING_STORAGE_TYPE");
+  storage_name = getenv("REDLAND_TEST_CLONING_STORAGE_NAME");
+  storage_options = getenv("REDLAND_TEST_CLONING_STORAGE_OPTIONS");
+
   if(!(storage_type && storage_name && storage_options)) {
     /* default is to test bdb disk hashes for cloning */
     storage_type="hashes";
     storage_name="test";
-    storage_options="hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'";
+#ifdef HAVE_BDB_HASH
+    storage_options = "hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'";
+#else
+    storage_options = "hash-type='memory',write='yes',new='yes',contexts='yes'";
+#endif
   }
 
   fprintf(stderr, "%s: Creating new %s storage\n", program, storage_type);
-  storage=librdf_new_storage(world, storage_type, storage_name, storage_options);
-  if(!storage) {
+  storage1 = librdf_new_storage(world, storage_type, storage_name, storage_options);
+  if(!storage1) {
     fprintf(stderr, "%s: Failed to create new %s storage name %s with options %s\n", program, storage_type, storage_name, storage_options);
     return 1;
   }
   
   fprintf(stderr, "%s: Creating new model for cloning\n", program);
-  model1=librdf_new_model(world, storage, NULL);
+  model1 = librdf_new_model(world, storage1, NULL);
   if(!model1) {
     fprintf(stderr, "%s: Failed to create new model\n", program);
-    /* ok to leak memory */    
-    return 1;
+    status = 1;
+    goto tidy;
   }
   
   fprintf(stderr, "%s: Cloning model\n", program);
-  model2=librdf_new_model_from_model(model1);
+  model2 = librdf_new_model_from_model(model1);
   if(!model2) {
     fprintf(stderr, "%s: Failed to clone model\n", program);
-    /* ok to leak memory */    
-    return 1;
+    status = 1;
+    goto tidy;
   }
+  storage2 = librdf_model_get_storage(model2);
 
   /* Free original model now so we can test whether the clone is self-sufficient */
   fprintf(stderr, "%s: Freeing original model\n", program);
-  librdf_free_model(model1);
+  librdf_free_model(model1); model1 = NULL;
 
   fprintf(stderr, "%s: Cloning cloned model\n", program);
-  model3=librdf_new_model_from_model(model2);
+  model3 = librdf_new_model_from_model(model2);
   if(!model3) {
     fprintf(stderr, "%s: Failed to clone cloned model\n", program);
-    /* ok to leak memory */    
-    return 1;
+    status = 1;
+    goto tidy;
   }
+  storage3 = librdf_model_get_storage(model3);
 
-  fprintf(stderr, "%s: Freeing cloned models\n", program);  
-  librdf_free_model(model3);
-  librdf_free_model(model2);
+  tidy:
+  fprintf(stderr, "%s: Freeing models\n", program);
+  if(model3)
+    librdf_free_model(model3);
+  if(model2)
+    librdf_free_model(model2);
+  if(model1)
+    librdf_free_model(model1);
   
-  fprintf(stderr, "%s: Freeing %s storage\n", program, storage_type);  
-  librdf_free_storage(storage);
-  
-  return 0;
+  fprintf(stderr, "%s: Freeing %s storages\n", program, storage_type);
+  if(storage3)
+    librdf_free_storage(storage3);
+  if(storage2)
+    librdf_free_storage(storage2);
+  librdf_free_storage(storage1);
+
+  return status;
 }
 
 #endif
