@@ -36,7 +36,7 @@
 #include <ctype.h>
 #include <sys/types.h>
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h> /* for abort() as used in errors */
+#include <stdlib.h>
 #endif
 
 #include <redland.h>
@@ -59,12 +59,24 @@ static void librdf_delete_query_factories(librdf_world *world);
  * compiled query modules.  Must be called before using any of the query
  * factory functions such as librdf_get_query_factory()
  **/
-void
+int
 librdf_init_query(librdf_world *world) 
 {
-  /* Always have query triple, rasqal implementations available */
-  librdf_query_triples_constructor(world);
-  librdf_query_rasqal_constructor(world);
+  int rc;
+  
+  /* Always have rasqal implementation available */
+  rc = librdf_query_rasqal_constructor(world);
+  if(rc)
+    return rc;
+
+#ifdef MODULAR_LIBRDF
+#else
+#ifdef STORAGE_VIRTUOSO
+  librdf_init_query_virtuoso(world);
+#endif
+#endif
+
+  return 0;
 }
 
 
@@ -153,12 +165,11 @@ librdf_query_register_factory(librdf_world *world, const char *name,
     }
   }
 
-  query=(librdf_query_factory*)LIBRDF_CALLOC(librdf_query_factory, 1,
-                                             sizeof(librdf_query_factory));
+  query = LIBRDF_CALLOC(librdf_query_factory*, 1, sizeof(*query));
   if(!query)
     goto oom;
 
-  query->name=(char*)LIBRDF_MALLOC(cstring, strlen(name)+1);
+  query->name = LIBRDF_MALLOC(char*, strlen(name) + 1);
   if(!query->name)
     goto oom_tidy;
   strcpy(query->name, name);
@@ -233,7 +244,77 @@ librdf_get_query_factory(librdf_world *world,
   return factory;
 }
 
+/**
+ * librdf_query_languages_enumerate:
+ * @world: #librdf_world
+ * @counter: index into the list of query language syntaxes
+ * @name: pointer to store the name of the query language syntax (or NULL)
+ * @uri_string: pointer to store query language syntax URI string (or NULL)
+ *
+ * Get information on query language syntaxes.
+ * 
+ * All returned strings are shared and must be copied if needed to be
+ * used dynamically.
+ *
+ * @deprecated: use librdf_query_language_get_description() to return
+ * more information in a static structure.
+ * 
+ * Return value: non 0 on failure of if counter is out of range
+ */
+int
+librdf_query_languages_enumerate(librdf_world* world,
+                                 const unsigned int counter,
+                                 const char **name,
+                                 const unsigned char **uri_string)
+{
+  unsigned int i;
+  librdf_query_factory *factory;
 
+  LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(world, librdf_world, 1);
+  if(!name && !uri_string)
+    return 1;
+
+  librdf_world_open(world);
+  
+  factory = world->query_factories;
+  if(!factory)
+    return 1;
+
+  for(i=0; factory && i<=counter ; i++, factory=factory->next) {
+    if(i == counter) {
+      if(name)
+        *name=factory->name;
+      if(uri_string) {
+        if (factory->uri)
+          *uri_string=librdf_uri_as_string(factory->uri);
+        else
+          *uri_string=NULL;
+      }
+      return 0;
+    }
+  }
+        
+  return 1;
+}
+
+/**
+ * librdf_query_language_get_description:
+ * @world: #librdf_world
+ * @counter: index into the list of query language syntaxes
+ *
+ * Get a query language syntax description.
+ *
+ * Return value: description or NULL if counter is out of range
+ */
+const raptor_syntax_description*
+librdf_query_language_get_description(librdf_world* world,
+                                      unsigned int counter)
+{
+  librdf_world_open(world);
+
+  return rasqal_world_get_query_language_description(world->rasqal_world_ptr,
+                                                     counter);
+}
 
 /**
  * librdf_new_query:
@@ -267,9 +348,11 @@ librdf_new_query(librdf_world *world,
 
 
 /**
- * librdf_new_query_from_query - Copy constructor - create a new librdf_query object from an existing one
+ * librdf_new_query_from_query:
  * @old_query: the existing query #librdf_query to use
  *
+ * Copy constructor - create a new librdf_query object from an existing one
+ *
  * Should create a new query in the same context as the existing one
  * as appropriate.
  *
@@ -288,15 +371,14 @@ librdf_new_query_from_query(librdf_query* old_query)
     return NULL;
   }
 
-  new_query=(librdf_query*)LIBRDF_CALLOC(librdf_query, 1, 
-                                         sizeof(librdf_query));
+  new_query = LIBRDF_CALLOC(librdf_query*, 1, sizeof(*new_query));
   if(!new_query)
     return NULL;
   
   new_query->usage=1;
 
-  new_query->context=(char*)LIBRDF_CALLOC(librdf_query_context, 1,
-                                          old_query->factory->context_length);
+  new_query->context = LIBRDF_CALLOC(void*, 1,
+                                     old_query->factory->context_length);
   if(!new_query->context) {
     librdf_free_query(new_query);
     return NULL;
@@ -352,7 +434,7 @@ librdf_new_query_from_factory(librdf_world *world,
     return NULL;
   }
   
-  query=(librdf_query*)LIBRDF_CALLOC(librdf_query, 1, sizeof(librdf_query));
+  query = LIBRDF_CALLOC(librdf_query*, 1, sizeof(*query));
   if(!query)
     return NULL;
 
@@ -360,8 +442,7 @@ librdf_new_query_from_factory(librdf_world *world,
 
   query->usage=1;
 
-  query->context=(char*)LIBRDF_CALLOC(librdf_query_context, 1,
-                                      factory->context_length);
+  query->context = LIBRDF_CALLOC(void*, 1, factory->context_length);
   if(!query->context) {
     librdf_free_query(query);
     return NULL;
@@ -387,8 +468,9 @@ librdf_new_query_from_factory(librdf_world *world,
 void
 librdf_free_query(librdf_query* query) 
 {
-  LIBRDF_ASSERT_OBJECT_POINTER_RETURN(query, librdf_query);
-
+  if(!query)
+    return;
+  
   if(--query->usage)
     return;
   
@@ -421,15 +503,17 @@ librdf_query_remove_query_result(librdf_query *query,
                                  librdf_query_results* query_results)
 {
   librdf_query_results *cur, *prev=NULL;
-  for(cur=query->results; cur && cur != query_results; cur=cur->next)
-    prev=cur;
+  for(cur = query->results; cur && cur != query_results; cur = cur->next)
+    prev = cur;
   
-  if(cur == query_results) {
-    if(prev)
-      prev->next=cur->next;
+  if(cur != NULL) {
+    if(cur == query_results) {
+      if(prev)
+        prev->next = cur->next;
+    }
+    if(cur == query->results)
+      query->results = cur->next;
   }
-  if(cur == query->results && cur != NULL)
-    query->results=cur->next;
 
   /* remove reference and free if we are the last */
   librdf_free_query(query);
@@ -569,7 +653,7 @@ ex:fido a ex:Dog ;\
 "
 #define DATA_LANGUAGE "turtle"
 #define DATA_BASE_URI "http://example.org/"
-#define QUERY_STRING "select ?x where (?x rdf:type ?y)";
+#define QUERY_STRING "select ?x where (?x rdf:type ?y)"
 #define QUERY_LANGUAGE "rdql"
 #define VARIABLES_COUNT 1
 
@@ -584,21 +668,39 @@ main(int argc, char *argv[])
   librdf_uri *uri;
   const char *program=librdf_basename((const char*)argv[0]);
   librdf_world *world;
-  librdf_uri* format_uri;
   size_t string_length;
   unsigned char *string;
   const char *query_string=QUERY_STRING;
-
+  int i;
+  
   world=librdf_new_world();
   librdf_world_open(world);
+  
+  /* list available query languages */
+  fprintf(stdout, "%s: Enumerating query languages:\n", program);
+  for (i = 0; 1; i++) {
+    const raptor_syntax_description* desc = NULL;
+    const char *name;
+    const unsigned char *uri;
+    
+    desc = librdf_query_language_get_description(world, i);
+    if (!desc)
+      break;
+      
+    fprintf(stdout, " %s <%s>\n", desc->names[0], desc->uri_strings_count ? desc->uri_strings[0] : "");
+  }
 
   /* create model and storage */
   if(1) {
     /* test in memory */
-    storage=librdf_new_storage(world, NULL, NULL, NULL);
+    storage = librdf_new_storage(world, NULL, NULL, NULL);
   } else {
     /* test on disk */
-    storage=librdf_new_storage(world, "hashes", "test", "hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'");
+#ifdef HAVE_BDB_HASH
+    storage = librdf_new_storage(world, "hashes", "test", "hash-type='bdb',dir='.',write='yes',new='yes',contexts='yes'");
+#else
+    storage = librdf_new_storage(world, "file", "test.rdf", NULL);
+#endif
   }
   if(!storage) {
     fprintf(stderr, "%s: Failed to create new storage\n", program);
@@ -677,17 +779,15 @@ main(int argc, char *argv[])
     return 1;
   }
 
-  format_uri=librdf_new_uri(world, (unsigned const char*)"http://www.w3.org/TR/2004/WD-rdf-sparql-XMLres-20041221/");
   string_length=0;
 
-  string=librdf_query_results_to_counted_string(results, 
-                                                format_uri, NULL,
-                                                &string_length);
+  string = librdf_query_results_to_counted_string2(results, 
+                                                   "xml", NULL, NULL,
+                                                   NULL, &string_length);
   fprintf(stdout, "%s: Got query results string of length %d\n", program,
           (int)string_length);
 
-  librdf_free_uri(format_uri);
-  free(string);
+  librdf_free_memory(string);
   
   librdf_free_query_results(results);
 
