@@ -36,7 +36,7 @@
 #include <ctype.h>
 #include <sys/types.h>
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h> /* for abort() as used in errors */
+#include <stdlib.h>
 #endif
 
 #ifdef MODULAR_LIBRDF
@@ -103,13 +103,8 @@ librdf_init_storage(librdf_world *world)
 #ifdef MODULAR_LIBRDF
 
   if (!world->storage_modules)
-#ifdef RAPTOR_V2_AVAILABLE
     world->storage_modules = raptor_new_sequence(
         (raptor_data_free_handler)lt_dlclose, NULL);
-#else
-    world->storage_modules = raptor_new_sequence(
-        (raptor_sequence_free_handler*)lt_dlclose, NULL);
-#endif
 
   librdf_storage_load_all_modules(world);
 
@@ -344,11 +339,8 @@ librdf_storage_register_factory(librdf_world* world,
 #endif
 
   if(!world->storages) {
-#ifdef RAPTOR_V2_AVAILABLE
     world->storages = raptor_new_sequence((raptor_data_free_handler)librdf_free_storage_factory, NULL);
-#else
-    world->storages = raptor_new_sequence((raptor_sequence_free_handler *)librdf_free_storage_factory, NULL);
-#endif
+
     if(!world->storages)
       goto failed;
   }
@@ -364,17 +356,16 @@ librdf_storage_register_factory(librdf_world* world,
     }
   }
 
-  storage=(librdf_storage_factory*)LIBRDF_CALLOC(librdf_storage_factory, 1,
-                                                 sizeof(librdf_storage_factory));
+  storage = LIBRDF_CALLOC(librdf_storage_factory*, 1, sizeof(*storage));
   if(!storage)
     goto failed;
 
-  storage->name=(char*)LIBRDF_MALLOC(cstring, strlen(name)+1);
+  storage->name = LIBRDF_MALLOC(char*, strlen(name) + 1);
   if(!storage->name)
     goto tidy;
   strcpy(storage->name, name);
 
-  storage->label=(char*)LIBRDF_MALLOC(cstring, strlen(label)+1);
+  storage->label = LIBRDF_MALLOC(char*, strlen(label) + 1);
   if(!storage->label)
     goto tidy;
   strcpy(storage->label, label);
@@ -503,9 +494,13 @@ librdf_new_storage(librdf_world *world,
   
   librdf_world_open(world);
 
-  factory=librdf_get_storage_factory(world, storage_name);
-  if(!factory)
+  factory = librdf_get_storage_factory(world, storage_name);
+  if(!factory) {
+    librdf_log(world, 
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "storage '%s' not found", name);
     return NULL;
+  }
 
   options_hash=librdf_new_hash(world, NULL);
   if(!options_hash)
@@ -549,9 +544,13 @@ librdf_new_storage_with_options(librdf_world *world,
   
   librdf_world_open(world);
 
-  factory=librdf_get_storage_factory(world, storage_name);
-  if(!factory)
+  factory = librdf_get_storage_factory(world, storage_name);
+  if(!factory) {
+    librdf_log(world, 
+               0, LIBRDF_LOG_ERROR, LIBRDF_FROM_STORAGE, NULL,
+               "storage '%s' not found", name);
     return NULL;
+  }
 
   options_hash=librdf_new_hash_from_hash(options);
   if(!options_hash)
@@ -597,8 +596,7 @@ librdf_new_storage_from_storage(librdf_storage* old_storage)
     return NULL;
   }
 
-  new_storage=(librdf_storage*)LIBRDF_CALLOC(librdf_storage, 1,
-                                             sizeof(librdf_storage));
+  new_storage = LIBRDF_CALLOC(librdf_storage*, 1, sizeof(*new_storage));
   if(!new_storage)
     return NULL;
   
@@ -659,8 +657,7 @@ librdf_new_storage_from_factory(librdf_world *world,
     return NULL;
   }
   
-  storage=(librdf_storage*)LIBRDF_CALLOC(librdf_storage, 1,
-                                         sizeof(librdf_storage));
+  storage = LIBRDF_CALLOC(librdf_storage*, 1, sizeof(*storage));
   if(!storage) {
     librdf_free_hash(options);
     return NULL;
@@ -1214,7 +1211,8 @@ librdf_storage_node_stream_to_node_create(librdf_storage* storage,
   if(!partial_statement)
     return NULL;
   
-  context=(librdf_storage_stream_to_node_iterator_context*)LIBRDF_CALLOC(librdf_storage_stream_to_node_iterator_context, 1, sizeof(librdf_storage_stream_to_node_iterator_context));
+  context = LIBRDF_CALLOC(librdf_storage_stream_to_node_iterator_context*, 1,
+                          sizeof(*context));
   if(!context) {
     librdf_free_statement(partial_statement);
     return NULL;
