@@ -34,7 +34,7 @@
 #include <stdio.h>
 #include <string.h>
 #ifdef HAVE_STDLIB_H
-#include <stdlib.h> /* for abort() as used in errors */
+#include <stdlib.h>
 #endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -138,6 +138,7 @@ static void librdf_storage_sqlite_find_statements_finished(void* context);
 /* context functions */
 static int librdf_storage_sqlite_context_add_statement(librdf_storage* storage, librdf_node* context_node, librdf_statement* statement);
 static int librdf_storage_sqlite_context_remove_statement(librdf_storage* storage, librdf_node* context_node, librdf_statement* statement);
+static int librdf_storage_sqlite_context_contains_statement(librdf_storage* storage, librdf_node* context, librdf_statement* statement);
 static librdf_stream* librdf_storage_sqlite_context_serialise(librdf_storage* storage, librdf_node* context_node);
 
 /* context sqlite statement stream methods */
@@ -184,9 +185,7 @@ librdf_storage_sqlite_init(librdf_storage* storage, const char *name,
     return 1;
   }
   
-  context = (librdf_storage_sqlite_instance*)LIBRDF_CALLOC(
-    librdf_storage_sqlite_instance, 1, sizeof(*context));
-
+  context = LIBRDF_CALLOC(librdf_storage_sqlite_instance*, 1, sizeof(*context));
   if(!context) {
     if(options)
       librdf_free_hash(options);
@@ -198,7 +197,7 @@ librdf_storage_sqlite_init(librdf_storage* storage, const char *name,
   context->storage = storage;
 
   context->name_len = strlen(name);
-  name_copy = (char*)LIBRDF_MALLOC(cstring, context->name_len + 1);
+  name_copy = LIBRDF_MALLOC(char*, context->name_len + 1);
   if(!name_copy) {
     if(options)
       librdf_free_hash(options);
@@ -224,7 +223,7 @@ librdf_storage_sqlite_init(librdf_storage* storage, const char *name,
       }
     }
     
-    LIBRDF_FREE(cstring, synchronous);
+    LIBRDF_FREE(char*, synchronous);
 
   }
   
@@ -248,7 +247,7 @@ librdf_storage_sqlite_terminate(librdf_storage* storage)
     return;
 
   if(context->name)
-    LIBRDF_FREE(cstring, context->name);
+    LIBRDF_FREE(char*, context->name);
   
   LIBRDF_FREE(librdf_storage_sqlite_terminate, storage->instance);
 }
@@ -342,7 +341,7 @@ sqlite_string_escape(const unsigned char *raw, size_t raw_len, size_t *len_p)
   }
 
   len = raw_len + escapes + 2; /* for '' */
-  escaped = (unsigned char*)LIBRDF_MALLOC(cstring, len + 1);
+  escaped = LIBRDF_MALLOC(unsigned char*, len + 1);
   if(!escaped)
     return NULL;
 
@@ -398,13 +397,11 @@ librdf_storage_sqlite_exec(librdf_storage* storage,
         sqlite_FREE(errmsg);
 
 
-      query = (librdf_storage_sqlite_query*)LIBRDF_CALLOC(
-        librdf_storage_sqlite_query, 1, sizeof(*query));
+      query = LIBRDF_CALLOC(librdf_storage_sqlite_query*, 1, sizeof(*query));
       if(!query)
         return 1;
 
-      query->query = (unsigned char*)LIBRDF_MALLOC(cstring,
-                                                   strlen((char *)request)+1);
+      query->query = LIBRDF_MALLOC(unsigned char*, strlen((char *)request) + 1);
       if(!query->query) {
         LIBRDF_FREE(librdf_storage_sqlite_query, query);
         return 1;
@@ -545,8 +542,7 @@ librdf_storage_sqlite_uri_helper(librdf_storage* storage,
   if(!uri_e)
     goto tidy;
   
-  expression = (unsigned char*)LIBRDF_MALLOC(cstring, 
-                                             strlen(field) + 3 + uri_e_len + 1);
+  expression = LIBRDF_MALLOC(unsigned char*, strlen(field) + 3 + uri_e_len + 1);
   if(!expression)
     goto tidy;
 
@@ -561,9 +557,9 @@ librdf_storage_sqlite_uri_helper(librdf_storage* storage,
 
   tidy:
   if(expression)
-    LIBRDF_FREE(cstring, expression);
+    LIBRDF_FREE(char*, expression);
   if(uri_e)
-    LIBRDF_FREE(cstring, uri_e);
+    LIBRDF_FREE(char*, uri_e);
 
   return id;
 }
@@ -586,8 +582,8 @@ librdf_storage_sqlite_blank_helper(librdf_storage* storage,
   if(!blank_e)
     goto tidy;
   
-  expression = (unsigned char*)LIBRDF_MALLOC(cstring,
-                                             strlen(field) + 3 + blank_e_len + 1);
+  expression = LIBRDF_MALLOC(unsigned char*,
+                             strlen(field) + 3 + blank_e_len + 1);
   if(!expression)
     goto tidy;
 
@@ -602,9 +598,9 @@ librdf_storage_sqlite_blank_helper(librdf_storage* storage,
 
   tidy:
   if(expression)
-    LIBRDF_FREE(cstring, expression);
+    LIBRDF_FREE(char*, expression);
   if(blank_e)
-    LIBRDF_FREE(cstring, blank_e);
+    LIBRDF_FREE(char*, blank_e);
 
   return id;
 }
@@ -696,9 +692,9 @@ librdf_storage_sqlite_literal_helper(librdf_storage* storage,
   if(sb)
     raptor_free_stringbuffer(sb);
   if(value_e)
-    LIBRDF_FREE(cstring, value_e);
+    LIBRDF_FREE(char*, value_e);
   if(language_e)
-    LIBRDF_FREE(cstring, language_e);
+    LIBRDF_FREE(char*, language_e);
   
   return id;
 }
@@ -994,10 +990,6 @@ static int
 librdf_storage_sqlite_add_statement(librdf_storage* storage, 
                                     librdf_statement* statement)
 {
-  /* Do not add duplicate statements */
-  if(librdf_storage_sqlite_contains_statement(storage, statement))
-    return 0;
-
   return librdf_storage_sqlite_context_add_statement(storage, NULL, statement);
 }
 
@@ -1037,7 +1029,7 @@ librdf_storage_sqlite_add_statements(librdf_storage* storage,
     }
 
     /* Do not add duplicate statements */
-    if(librdf_storage_sqlite_contains_statement(storage, statement))
+    if(librdf_storage_sqlite_context_contains_statement(storage, context_node, statement))
       continue;
 
     if(librdf_storage_sqlite_statement_helper(storage,
@@ -1172,6 +1164,15 @@ librdf_storage_sqlite_statement_operator_helper(librdf_storage* storage,
 static int
 librdf_storage_sqlite_contains_statement(librdf_storage* storage, 
                                          librdf_statement* statement)
+{
+  return librdf_storage_sqlite_context_contains_statement(storage, NULL, statement);
+}
+
+
+static int
+librdf_storage_sqlite_context_contains_statement(librdf_storage* storage,
+                                                 librdf_node* context_node,
+                                                 librdf_statement* statement)
 {
   raptor_stringbuffer *sb;
   unsigned char *request;
@@ -1190,7 +1191,7 @@ librdf_storage_sqlite_contains_statement(librdf_storage* storage,
                                     1);
 
   if(librdf_storage_sqlite_statement_operator_helper(storage, statement, 
-                                                     NULL, sb, 0)) {
+                                                     context_node, sb, 0)) {
     if(!begin)
       librdf_storage_sqlite_transaction_rollback(storage);
     raptor_free_stringbuffer(sb);
@@ -1201,10 +1202,10 @@ librdf_storage_sqlite_contains_statement(librdf_storage* storage,
   request = raptor_stringbuffer_as_string(sb);
   
   rc = librdf_storage_sqlite_exec(storage,
-                                request,
-                                librdf_storage_sqlite_get_1int_callback,
-                                &count,
-                                0);
+                                  request,
+                                  librdf_storage_sqlite_get_1int_callback,
+                                  &count,
+                                  0);
   
   raptor_free_stringbuffer(sb);
 
@@ -1288,8 +1289,8 @@ librdf_storage_sqlite_serialise(librdf_storage* storage)
   
   context = (librdf_storage_sqlite_instance*)storage->instance;
 
-  scontext = (librdf_storage_sqlite_serialise_stream_context*)LIBRDF_CALLOC(
-    librdf_storage_sqlite_serialise_stream_context, 1, sizeof(*scontext));
+  scontext = LIBRDF_CALLOC(librdf_storage_sqlite_serialise_stream_context*,
+                           1, sizeof(*scontext));
   if(!scontext)
     return NULL;
 
@@ -1392,7 +1393,6 @@ librdf_storage_sqlite_get_next_common(librdf_storage_sqlite_instance* scontext,
 #endif
     if(status == SQLITE_BUSY) {
       /* FIXME - how to handle busy? */
-      status = SQLITE_ERROR;
       continue;
     }
     break;
@@ -1725,8 +1725,8 @@ librdf_storage_sqlite_find_statements(librdf_storage* storage,
   
   context = (librdf_storage_sqlite_instance*)storage->instance;
 
-  scontext = (librdf_storage_sqlite_find_statements_stream_context*)LIBRDF_CALLOC(
-    librdf_storage_sqlite_find_statements_stream_context, 1, sizeof(*scontext));
+  scontext = LIBRDF_CALLOC(librdf_storage_sqlite_find_statements_stream_context*,
+                           1, sizeof(*scontext));
   if(!scontext)
     return NULL;
 
@@ -1990,6 +1990,10 @@ librdf_storage_sqlite_context_add_statement(librdf_storage* storage,
   int rc, begin;
   int max=3;
 
+  /* Do not add duplicate statements */
+  if(librdf_storage_sqlite_context_contains_statement(storage, context_node, statement))
+    return 0;
+
   /* context = (librdf_storage_sqlite_instance*)storage->instance; */
 
   sb = raptor_new_stringbuffer();
@@ -2210,8 +2214,8 @@ librdf_storage_sqlite_context_serialise(librdf_storage* storage,
 
   context = (librdf_storage_sqlite_instance*)storage->instance;
 
-  scontext = (librdf_storage_sqlite_context_serialise_stream_context*)LIBRDF_CALLOC(
-    librdf_storage_sqlite_context_serialise_stream_context, 1, sizeof(*scontext));
+  scontext = LIBRDF_CALLOC(librdf_storage_sqlite_context_serialise_stream_context*,
+                           1, sizeof(*scontext));
   if(!scontext)
     return NULL;
 
@@ -2424,6 +2428,9 @@ librdf_storage_sqlite_context_serialise_finished(void* context)
   if(scontext->context)
     librdf_free_node(scontext->context);
 
+  if(scontext->context_node)
+    librdf_free_node(scontext->context_node);
+
   scontext->sqlite_context->in_stream--;
   if(!scontext->sqlite_context->in_stream)
     librdf_storage_sqlite_query_flush(scontext->storage);
@@ -2476,7 +2483,6 @@ librdf_storage_sqlite_get_next_context_common(librdf_storage_sqlite_instance* sc
 #endif
     if(status == SQLITE_BUSY) {
       /* FIXME - how to handle busy? */
-      status = SQLITE_ERROR;
       continue;
     }
     break;
@@ -2694,8 +2700,8 @@ librdf_storage_sqlite_get_contexts(librdf_storage* storage)
 
   context = (librdf_storage_sqlite_instance*)storage->instance;
 
-  icontext = (librdf_storage_sqlite_get_contexts_iterator_context*)LIBRDF_CALLOC(
-    librdf_storage_sqlite_get_contexts_iterator_context, 1, sizeof(*icontext));
+  icontext = LIBRDF_CALLOC(librdf_storage_sqlite_get_contexts_iterator_context*,
+                           1, sizeof(*icontext));
   if(!icontext)
     return NULL;
 
@@ -2917,7 +2923,7 @@ librdf_storage_sqlite_query_flush(librdf_storage *storage)
 
     librdf_storage_sqlite_exec(storage, query->query, NULL, NULL, 0);
 
-    LIBRDF_FREE(cstring, query->query);
+    LIBRDF_FREE(char*, query->query);
     LIBRDF_FREE(librdf_storage_sqlite_query, query);
   }
 
